<?php

namespace AustraliaPost\Extensions\Business\API\Wrapper;

use AustraliaPost\Extensions\Business\Helpers\Utilities;

class Label {
	private $type = "PRINT";
	private $format = "PDF";
	private $groups = [];
	private $shipments = [];
	private $isBranded = true;

	const LAYOUT_A4_1PP = 'A4-1pp';
	const LAYOUT_A4_2PP = 'A4-2pp';
	const LAYOUT_A4_3PP = 'A4-3pp';
	const LAYOUT_A4_4PP = 'A4-4pp';
	const LAYOUT_THERMAL_LABEL_A6_1PP = 'THERMAL-LABEL-A6-1PP';

	/**
	 * @return string
	 */
	public function getType() {
		return $this->type;
	}

	/**
	 * @param string $type
	 */
	public function setType( $type ) {
		$this->type = $type;
	}

	/**
	 * @return string
	 */
	public function getFormat() {
		return $this->format;
	}

	/**
	 * @param string $format
	 */
	public function setFormat( $format ) {
		$this->format = $format;
	}

	/**
	 * @return array
	 */
	public function getGroups() {
		return $this->groups;
	}

	/**
	 *
	 * Sample group
	 *
	 * - "group": "Parcel Post"
	 * - "layout": "A4-1pp"
	 * - "branded": true
	 * - "left_offset": 0
	 * - "top_offset": 0
	 * @param array $group
	 */
	public function addGroup(  $group )
	{
		$group_defaults = [
			'branded' => $this->isBranded(),
			'left_offset' => apply_filters('labels_pro_label_left_offset', 0),
			'top_offset' => apply_filters('labels_pro_label_top_offset', 0),
		];

		$this->groups[] = array_merge($group, $group_defaults);
	}

	/**
	 * @return array
	 */
	public function getShipments() {
		return $this->shipments;
	}

	/**
	 * "shipment_id"
	 * "items" array of "item_id"
	 * @param array $shipment
	 */
	public function addShipment( $shipment ) {
		$this->shipments[] = $shipment;
	}

	private function getPreferences() {
		return [
			'type'  =>  $this->getType(),
			'format'  =>  $this->getFormat(),
			'groups'    => $this->getGroups()
		];
	}

	public function getPayLoad(){
		$payload = [
			'wait_for_label_url' => true,
			'preferences'   => [$this->getPreferences()],

		];
		foreach ($this->getShipments() as $shipment) {
			$payloadShipment = [
				'shipment_id' => $shipment['shipment_id'],
				'items' => [],
			];
			if (isset($shipment['items']) && is_array($shipment['items'])) {
				foreach ($shipment['items'] as $item) {
					$itemPayload = [ 'item_id' => $item['item_id'] ];
					$payloadShipment['items'][] = $itemPayload;
				}
			}
			$payload['shipments'][] = $payloadShipment;
		}

		return $payload;
	}

	public static function available_layouts()
	{
		return [
			self::LAYOUT_A4_1PP => 'A4 - 1 Label Per Page',
			self::LAYOUT_A4_2PP => 'A4 - 2 Labels Per Page',
			self::LAYOUT_A4_3PP => 'A4 - 3 Labels Per Page',
			self::LAYOUT_A4_4PP => 'A4 - 4 Labels Per Page',
			self::LAYOUT_THERMAL_LABEL_A6_1PP => 'Thermal A6 - 1 Label Per Page',
		];
	}

	public function setLabelLayouts( $label_layout ) {
		$parcel_layouts = [self::LAYOUT_A4_4PP, self::LAYOUT_A4_1PP, self::LAYOUT_THERMAL_LABEL_A6_1PP];

		if (Utilities::isMyPost()) {
			$express_layouts = [self::LAYOUT_A4_4PP, self::LAYOUT_A4_1PP, self::LAYOUT_THERMAL_LABEL_A6_1PP];
		} else {
			$express_layouts = [self::LAYOUT_A4_4PP, self::LAYOUT_A4_3PP, self::LAYOUT_A4_1PP, self::LAYOUT_THERMAL_LABEL_A6_1PP];
		}

		$international_layouts = [self::LAYOUT_A4_4PP, self::LAYOUT_A4_1PP, self::LAYOUT_THERMAL_LABEL_A6_1PP];

		$this->addGroup( [
			'group' => 'Parcel Post',
			'layout' => in_array($label_layout, $parcel_layouts)? $label_layout: self::LAYOUT_A4_4PP,
		]);

		$this->addGroup( [
			'group' => 'Express Post',
			'layout' => in_array($label_layout, $express_layouts)? $label_layout: self::LAYOUT_A4_4PP,
		]);

		$this->addGroup( [
			'group' => 'International',
			'layout' => in_array($label_layout, $international_layouts)? $label_layout: self::LAYOUT_A4_1PP,
		]);
	}

	/**
	 * @return bool
	 */
	public function isBranded() {
		return $this->isBranded;
	}

	/**
	 * @param bool $isBranded
	 *
	 * @return Label
	 */
	public function setIsBranded( $isBranded ) {
		$this->isBranded = $isBranded;

		return $this;
	}


}
