<?php
namespace AustraliaPost\Extensions\Business\API\Wrapper;

use AustraliaPost\Extensions\Business\Helpers\Utilities;

class Price_Item{

	private $item_reference;
	private $product_id;
	private $product_ids = [];
	private $authority_to_leave = false;
	private $allow_partial_delivery = true;
	private $signature_on_delivery = false;
	private $contains_dangerous_goods = false;
	private $non_delivery_action = 'RETURN';
	private $length;
	private $height;
	private $width;
	private $weight;
	private $features = [];
	private $total_products_price;
	private $item_contents = [];
	private $accountType = 'eparcel';

	const CLASSIFICATION_TYPE_GIFT = 'GIFT';
	const CLASSIFICATION_TYPE_SAMPLE = 'SAMPLE';
	const CLASSIFICATION_TYPE_DOCUMENT = 'DOCUMENT';
	const CLASSIFICATION_TYPE_RETURN = 'RETURN';
	const CLASSIFICATION_TYPE_SALE_OF_GOODS = 'SALE_OF_GOODS';

	const FLAT_RATE_PRODUCTS = [
		'BE1P05', 'BE9P30', 'BE9P50',
		'BE1P30', 'BE1P50', 'BE9P10',
		'BE1P10', 'BE9P05', 'BE1PB1',
		'BE1PB2', 'BE1PB3', 'BE1PB4',
		'BE9PB1', 'BE9PB2', 'BE9PB3',
		'BE9PB4'
	];

	/**
	 * @param string $accountType
	 */
	public function __construct( $accountType = 'eparcel' ) {
		$this->accountType = $accountType;
	}


	/**
	 * @return mixed
	 */
	public function getLength() {
		return round($this->length, 1);
	}

	/**
	 * @param mixed $length
	 *
	 * @return Price_Item
	 */
	public function setLength( $length ) {
		$this->length = $length;
		return $this;
	}

	/**
	 * @return mixed
	 */
	public function getHeight() {
		return round($this->height, 1);
	}

	/**
	 * @param mixed $height
	 *
	 * @return Price_Item
	 */
	public function setHeight( $height ) {
		$this->height = $height;
		return $this;

	}

	/**
	 * @return mixed
	 */
	public function getWidth() {
		return round($this->width, 1);

	}

	/**
	 * @param mixed $width
	 *
	 * @return Price_Item
	 */
	public function setWidth( $width ) {
		$this->width = $width;
		return $this;
	}

	/**
	 * @return mixed
	 */
	public function getWeight() {
		return round($this->weight, 2);
	}

	/**
	 * @param mixed $weight
	 *
	 * @return Price_Item
	 */
	public function setWeight( $weight ) {
		$this->weight = $weight;
		return $this;
	}

	/**
	 * @return mixed
	 */
	public function getItemReference() {
		return $this->item_reference;
	}

	/**
	 * @param mixed $item_reference
	 *
	 * @return Price_Item
	 */
	public function setItemReference( $item_reference ) {
		$this->item_reference = $item_reference;
		return $this;
	}

	/**
	 * @return array<Price_Feature>
	 */
	public function getFeatures()
	{
		return $this->features;
	}

	/**
	 * @param Price_Feature $feature
	 *
	 * @return Price_Item
	 */
	public function addFeature( $feature )
	{
		$this->features[] = $feature;
		return $this;
	}

	/**
	 * @return bool
	 */
	public function isContainsDangerousGoods(): bool
	{
		return $this->contains_dangerous_goods;
	}

	/**
	 * @param bool $contains_dangerous_goods
	 *
	 * @return Price_Item
	 */
	public function setContainsDangerousGoods( bool $contains_dangerous_goods ): Price_Item
	{
		$this->contains_dangerous_goods = $contains_dangerous_goods;

		return $this;
	}

	public function getPayLoad($toCountry)
	{
		$payload = [];
		if (Utilities::isEParcel()) {
			$payload['length'] = $this->getLength();
			$payload['width'] = $this->getWidth();
			$payload['height'] = $this->getHeight();
			$payload['weight'] = $this->getWeight();
		} elseif (Utilities::isMyPost()) {

			if ($toCountry !== 'AU') {
				$payload['non_delivery_action'] = $this->getNonDeliveryAction();
			}

			if (count($this->getProductIds()) > 0) {
				if(count(array_intersect($this->getProductIds(), self::FLAT_RATE_PRODUCTS)) === 0) {
					$payload['length'] = $this->getLength();
					$payload['width'] = $this->getWidth();
					$payload['height'] = $this->getHeight();
					$payload['weight'] = $this->getWeight();
				}
			} elseif(!in_array($this->getProductId(), self::FLAT_RATE_PRODUCTS)) {
				$payload['length'] = $this->getLength();
				$payload['width'] = $this->getWidth();
				$payload['height'] = $this->getHeight();
				$payload['weight'] = $this->getWeight();
			}
		}

		$payload['item_reference'] = $this->getItemReference();

		if(count($this->getProductIds()) > 0) {
			$payload['product_ids'] = $this->getProductIds();
		} else {
			$payload['product_id'] = $this->getProductId();
		}

		if ($toCountry === 'AU') {
			if ($this->accountType === 'eparcel') {
				$payload['allow_partial_delivery'] = false;
				$payload['authority_to_leave'] = !$this->getSignatureOnDelivery();
				$payload['safe_drop_enabled'] = !$this->getSignatureOnDelivery();
			}
		} else {
			$payload = $this->add_international_clearance_info($payload);
		}

		if ($this->getSignatureOnDelivery()) {
			$payload['signature_on_delivery'] = $this->signature_on_delivery;
		}

		if ($this->isContainsDangerousGoods()) {
			$payload['contains_dangerous_goods'] = true;
		}

		/** @var Price_Feature $feature */
		if(is_array($this->getFeatures())){
			foreach ( $this->getFeatures() as $feature){
				$payload['features'][$feature->getCode()] = $feature->getPayload();
			}
		}

		return $payload;
	}

	/**
	 * @return mixed
	 */
	public function getProductId() {
		return $this->product_id;
	}

	/**
	 * @param mixed $product_id
	 * @return Price_Item
	 */
	public function setProductId( $product_id ) {
		$this->product_id = $product_id;
		return $this;
	}

	/**
	 * @return bool
	 */
	public function isAuthorityToLeave() {
		return $this->authority_to_leave;
	}

	/**
	 * @param bool $authority_to_leave
	 */
	public function setAuthorityToLeave( $authority_to_leave ) {
		$this->authority_to_leave = $authority_to_leave;
	}

	/**
	 * @return bool
	 */
	public function isAllowPartialDelivery() {
		return $this->allow_partial_delivery;
	}

	/**
	 * @param bool $allow_partial_delivery
	 */
	public function setAllowPartialDelivery( $allow_partial_delivery ) {
		$this->allow_partial_delivery = $allow_partial_delivery;
	}

	/**
	 * @return mixed
	 */
	public function getTotalProductsPrice() {
		return $this->total_products_price;
	}

	/**
	 * @param mixed $total_products_price
	 * @return Price_Item
	 */
	public function setTotalProductsPrice( $total_products_price ) {
		$this->total_products_price = $total_products_price;
		return $this;
	}

	/**
	 * @return bool
	 */
	public function getSignatureOnDelivery() {
		return $this->signature_on_delivery;
	}

	/**
	 * @param bool $signature_on_delivery
	 */
	public function setSignatureOnDelivery( $signature_on_delivery ) {
		$this->signature_on_delivery = $signature_on_delivery;
	}

	/**
	 * @param $payload
	 *
	 * @return mixed
	 */
	private function add_international_clearance_info( $payload ) {

		$payload['classification_type'] = self::CLASSIFICATION_TYPE_SALE_OF_GOODS;
		$payload['commercial_value'] = false;

		/** @var Price_Item_Clearance_Description $itemContent */
		foreach ($this->getItemContents() as $key => $itemContent) {
			$payload['item_contents'][$key] = $itemContent->getPayload();
		}

		return $payload;
	}

	/**
	 * @return array<Price_Item_Clearance_Description>
	 */
	public function getItemContents() {
		return $this->item_contents;
	}

	/**
	 * @param Price_Item_Clearance_Description $item_content
	 */
	public function addItemContent( $item_content ) {
		$this->item_contents[] = $item_content;
	}

	/**
	 * @return array
	 */
	public function getProductIds() {
		return $this->product_ids;
	}

	/**
	 * @param array $product_ids
	 *
	 * @return Price_Item
	 */
	public function setProductIds( array $product_ids ): Price_Item {
		$this->product_ids = $product_ids;

		return $this;
	}

	/**
	 * @return string
	 */
	public function getNonDeliveryAction()
	{
		return $this->non_delivery_action;
	}

	/**
	 * @param string $non_delivery_action
	 *
	 * @return Price_Item
	 */
	public function setNonDeliveryAction( $non_delivery_action )
	{
		$this->non_delivery_action = $non_delivery_action;

		return $this;
	}




}
